require 'shellwords'
require 'tempfile'

require 'swf_converter/engine'

module SwfConverter
  class Engine
    class Alphize < Engine
      SWF_PATHS = {}.tap do |paths|
        [:black, :white].each do |type|
          paths[type] = File.expand_path("../alphize/swfs/#{type}.swf", __FILE__)
        end
      end

      def convert!(swf_parent)
        images = {
          :black => Tempfile.open('swf_converter_alphize_black.png'),
          :white => Tempfile.open('swf_converter_alphize_white.png')
        }
        begin
          write_opaque_images(swf_parent, images)
          blend(swf_parent, images)
        ensure
          images[:black].close!
          images[:white].close!
        end
      end

      protected

      def write_opaque_images(swf_parent, images)
        raw = Tempfile.open 'swf_converter_alphize_raw'
        raw_path = e(raw.path)
        size = @options[:size]
        unless size
          raise ":size option must be set to the original SWF dimensions"
        end
        # Each RGBA pixel takes a byte, so the filesize of one frame is the
        # number of pixels (width * height) times the number of channels (4)
        frame_filesize = size[0] * size[1] * 4
        begin
          images.each do |type, image|
            wrapper_swf = e(SWF_PATHS[type])
            params = e("FlashVars=includeSWF=file://#{swf_parent.local_swf_path}")
            image_path = e(image.path)
            run "create raw SWF dump on #{type}",
              "dump-gnash #{wrapper_swf} -P #{params} " +
              "-D #{raw_path} --max-advances 1 -j #{size[0]} -k #{size[1]} > /dev/null"
            run "create opaque image from raw on #{type}",
              "tail -c #{frame_filesize} #{raw_path} | convert -size #{size[0]}x#{size[1]} -depth 8 rgba:- " +
              "-separate -swap 0,2 -combine png:#{image_path}"
          end
        ensure
          raw.close!
        end
      end

      def blend(swf_parent, images)
        begin
          blender = Blender.new(images)
          if @options[:sizes]
            blend_and_resize(swf_parent, blender)
          else
            blender.blend! swf_parent.swf_image_path
          end
        rescue Exception => e
          raise ConversionError, "Could not blend opaque images: #{e.class}: #{e.message}", e.backtrace
        end
        blender = nil
        GC.start
      end

      def blend_and_resize(swf_parent, blender)
        original_image = Tempfile.new('swf_converter_alphize_original.png')
        original_path = original_image.path
        begin
          blender.blend! original_path
          @options[:sizes].each do |size|
            resize(swf_parent, size, original_path)
          end
        ensure
          original_image.close!
        end
      end

      def resize(swf_parent, size, original_path)
        output_path = swf_parent.swf_image_path(size)
        run "resize #{original_path} to #{size} at #{output_path}",
          "convert #{original_path} -resize #{size[0]}x#{size[1]} #{output_path}"
      end

      def run(description, command)
        unless system(command)
          raise ConversionError, "Failed to #{description}: #{command.inspect}"
        end
      end

      def e(*args)
        Shellwords.escape(*args)
      end

      def validate_options!(options)
        if options[:size]
          validate_size(options[:size], ":size option")
        end

        if options[:output_sizes]
          options[:output_sizes].each_with_index do |size, n|
            validate_size(size, "Element #{n} of :output_sizes option")
          end
        end
      end

      def validate_size(size, description)
        unless size.is_a?(Array) && size.size == 2
          raise ArgumentError, "#{description} must be a 2-dimensional array, but was #{size}"
        end
      end

      class ConversionError < StandardError;end
    end
  end
end

