require 'chunky_png'
require 'oily_png'

module SwfConverter
  class Engine
    class Alphize < Engine
      class Blender
        def initialize(image_files)
          @images = {}
          sizes = {}
          image_files.each do |type, image_file|
            @images[type] = ChunkyPNG::Image.from_io(image_file)
            sizes[type] = [@images[type].width, @images[type].height]
          end
          if sizes[:black] != sizes[:white]
            raise RuntimeError, "Image on black is size #{sizes[:black]}, whereas image on white is size #{sizes[:white]}"
          end
          @size = sizes[:black]
        end

        def blend!(output_path)
          blended_image.save(output_path)
        end

        protected

        def blended_image
          image = ChunkyPNG::Image.new(*@size)
          (0...@size[0]).each do |x|
            (0...@size[1]).each do |y|
              image[x, y] = blend_colors_at(x, y)
            end
          end
          image
        end

        def blend_colors_at(x, y)
          color_on_black = @images[:black][x, y]
          color_on_white = @images[:white][x, y]

          if color_on_black == color_on_white
            original_color = color_on_black
          elsif color_on_black == ChunkyPNG::Color::BLACK && color_on_white == ChunkyPNG::Color::WHITE
            original_color = ChunkyPNG::Color::TRANSPARENT
          else
            alpha = ChunkyPNG::Color::r(color_on_black) - ChunkyPNG::Color::r(color_on_white) + ChunkyPNG::Color::MAX
            original_color = ChunkyPNG::Color::fade(color_on_black, alpha)
          end
          original_color
        end
      end
    end
  end
end

