require 'spec_helper'
require 'swf_converter/engine/alphize'

require 'tempfile'

describe SwfConverter::Engine::Alphize do
  before(:each) do
    @engine = SwfConverter::Engine::Alphize.new
  end

  describe "size option" do
    it "should be settable" do
      @engine.options = {:size => [300, 500]}
      @engine.options[:size].should == [300, 500]
    end

    it "should require a 2-dimensional array" do
      lambda { @engine.options = {:size => [300, 300]} }.should_not raise_error
      lambda { @engine.options = {:size => [7, 8]} }.should_not raise_error
      lambda { @engine.options = {:size => 300} }.should raise_error
      lambda { @engine.options = {:size => [300]} }.should raise_error
      lambda { @engine.options = {:size => [300, 300, 300]} }.should raise_error
      lambda { @engine.options = {:size => [300, 300, 300, 300]} }.should raise_error
    end
  end

  describe "output_sizes option" do
    it "should be settable" do
      @engine.options = {:output_sizes => [[300, 300]]}
      @engine.options[:output_sizes].should == [[300, 300]]
    end

    it "should require all elements to be 2-dimensional arrays" do
      lambda { @engine.options = {:output_sizes => [[300, 300]]} }.should_not raise_error
      lambda { @engine.options = {:output_sizes => [[300, 300], [7, 8]]} }.should_not raise_error
      lambda { @engine.options = {:output_sizes => [[300, 300], [7, 8], [1026, 1993]]} }.should_not raise_error
      lambda { @engine.options = {:output_sizes => 300} }.should raise_error
      lambda { @engine.options = {:output_sizes => [300]} }.should raise_error
      lambda { @engine.options = {:output_sizes => [[300]]} }.should raise_error
      lambda { @engine.options = {:output_sizes => [300, 300]} }.should raise_error
      lambda { @engine.options = {:output_sizes => [[150, 150], [300]]} }.should raise_error
      lambda { @engine.options = {:output_sizes => [[300, 300], [150, 150], [10, 26, 1993]]} }.should raise_error
    end
  end

  describe "with a size of 600x600 set" do
    before(:each) do
      @engine.options = {:size => [600, 600]}
    end

    it "should convert the balloon to a single 600x600 image" do
      output_file = Tempfile.new 'balloon.png'
      begin
        parent = mock('balloon', :local_swf_path => example_path('balloon.swf'),
          :swf_image_path => output_file.path)
        @engine.convert! parent
        output_file.should match_image(example_path('balloon.png'))
      ensure
        output_file.close!
      end
    end

    it "should convert the rainy day to a single 600x600 image" do
      output_file = Tempfile.new 'rainy_day.png'
      begin
        parent = mock('rainy_day', :local_swf_path => example_path('rainy_day.swf'),
          :swf_image_path => output_file.path)
        @engine.convert! parent
        output_file.should match_image(example_path('rainy_day.png'))
      ensure
        output_file.close!
      end
    end

    describe "and 80x80 and 150x150 output sizes set" do
      before(:each) do
        @engine.options[:sizes] = [
          [80, 80],
          [150, 150]
        ]
      end

      it "should convert the balloon to 80x80 and 150x150 thumbnails" do
        thumbnails = {
          80 => Tempfile.new('balloon80.png'),
          150 => Tempfile.new('balloon150.png')
        }
        parent = mock('balloon_thumbnails', :local_swf_path => example_path('balloon.swf'))
        parent.stub!(:swf_image_path).with([80, 80]).and_return(thumbnails[80].path)
        parent.stub!(:swf_image_path).with([150, 150]).and_return(thumbnails[150].path)
        @engine.convert! parent
        thumbnails[80].should match_image(example_path('balloon/80x80.png'))
        thumbnails[150].should match_image(example_path('balloon/150x150.png'))
      end
    end
  end
end

