require 'spec_helper'
require 'swf_converter/engine'

describe SwfConverter::Engine do
  describe "#options" do
    it "should return the options set" do
      engine = SwfConverter::Engine.new
      engine.options = {:foo => :bar}
      engine.options.should == {:foo => :bar}
    end
    
    it "should allow a subclass to validate options on set" do
      lambda {
        engine = ValidatingEngine.new
        engine.options = {:foo => :bar}
      }.should raise_error(SwfConverter::InvalidOptions)
      
      lambda {
        engine = ValidatingEngine.new
        engine.options = {:foo => :bar, :sizes => [150, 300]}
      }.should_not raise_error
    end
  end
  
  describe "#convert!" do
    it "should raise an error if not implemented by subclass" do
      parent = mock('engine_conversion_parent')
      
      lambda {
        engine = SwfConverter::Engine.new
        engine.convert!(parent)
      }.should raise_error(NotImplementedError)
      
      lambda {
        engine = SupposedlyConvertingEngine.new
        engine.convert!(parent)
      }.should_not raise_error
    end
  end
end
